<?php
// ===================================================
//               X.able PHP CMS module
//           (C)2016 maciej@maciejnowak.com
// ===================================================
// compatibility: php4+
// build: 20161021
// ===================================================

    function getCmsUrl() {
    // -----------------------------------
    // RETURNS: <string> CMS directory URL
    // -----------------------------------
        return $_SERVER['SERVER_NAME'].dirname($_SERVER['PHP_SELF'])."/";
    };

// ===================================================
//                     LOCALIZE
// ===================================================
        
    function localize($key) {
    // -----------------------------------
    // $key = <string>
    // -----------------------------------
    // RETURNS: <string> Localized text
    // -----------------------------------
        $val_1 = $_SESSION['localize'][$key];
        $val_2 = $_SESSION['localize'][$key][ $_SESSION["admin_lang"] ];
        if(is_string($val_1) && $val_1 != "") {
            return $val_1;
        }
        if(is_string($val_2) && $val_2 != "") {
            return $val_2;
        }
        else {
            return $key;
        }
    };

    function exportLocalization() {
    // -----------------------------------
    // Export localization data to js
    // -----------------------------------
        $variable_list = [ "dictionary", "localize" ];
        echo "\n";
        echo "\t\t<script>\n";
        foreach($variable_list as $variable) {
            $json = [];
            foreach(array_keys($_SESSION[$variable]) as $key) {
                $text = $_SESSION[$variable][$key];
                $text = str_replace("\"", "\\\"", $text);
                $text = str_replace("&#39;", "\u0027", $text);
                $text = str_replace("[br]", "<br>", $text);
                $json[] = "\"".$key."\": \"".$text."\"";
            }
            echo "\t\t\tvar ".strtoupper($variable)." = { ".join(", ", $json)." };\n";
        }
        echo "\t\t</script>\n";
    };

    function loadLocalization($folder, $language = false) {
    // -----------------------------------
    // $folder = <string> localization folder path
    // $language = <string> Selected language (optional)
    // -----------------------------------
    // Load localization data to $_SESION variables
    // -----------------------------------
        if(!$language) { $language = $_SESSION["admin_lang"]; };
        $translations = [ "localize" => [], "dictionary" => [] ];
        if(file_exists("$folder/$language.json")) {
            $json = file_get_contents("$folder/$language.json");
            $array = json_decode($json, true);
            foreach(array_keys($array) as $folder) {
                if(substr($folder, 0, 1) == "_") { $group = "dictionary"; } else { $group = "localize"; };
                $content = $translations[$group];
                $translations[$group] = array_merge($content, $array[$folder]);
            }
        }
        foreach(array_keys($translations) as $group) {
            $_SESSION[$group] = $translations[$group];
        }
    };

// ===================================================
//                        LOG
// ===================================================

    function addLog($action, $filepath) {
    // -----------------------------------
    // $action = <string>
    // $filepath = <string>
    // -----------------------------------
    // Add saving changes info to zable.log file -> time, action, document path, group.user
    // -----------------------------------
        $ini_pathes = loadIni("xable.ini", "pathes");
        $log_file = $ini_pathes['log'];
        $log = array(
                date("Y.m.d G:i:s"),    
                $filepath,
                $action,
                $_SESSION['logged_group'].".".$_SESSION['logged_user']
            );
        if(file_exists($log_file)) {
            $log_list = array_map("trim", file($log_file));
            $title_row = array_shift($log_list);
        }
        else {
            $log_list = [];
            $title_row = "Czas;Dokument;Akcja;Użytkownik";
        };
        // Log max length
        $ini_pathes = loadIni("xable.ini", "options");
        $log_max = $ini_pathes['log_max'];
        if(is_numeric($log_max) && count($log_list) > $log_max) {
            $log_list = array_reverse(array_slice(array_reverse($log_list), 0, $log_max));
        };
        // Output
        $log_list[] = join(";", $log);
        array_unshift($log_list, $title_row);
        safeSave($log_file, join("\n", $log_list));
    };

// ===================================================
//                        INI
// ===================================================

    function loadIni($path, $type) {
    // --------------------------------
    // $path = <string> ini file path
    // $type = <string> data type to read: "navigation", "pathes"
    // --------------------------------
    // RETURNS" <array> CMS content data
    // --------------------------------
        if(file_exists($path)) {
            
            $file = array_map("trim", file($path));
            $ini = [];
            $label = false;
            foreach(array_keys($file) as $n) {        
                $line = trim($file[$n]);
                // ====== Navigation ======
                if($type == "navigation") {
                    if(substr($line, 0, 1) == ";" || $line == "") {} // comments;
                    elseif(preg_match("/^\[nav:(.*?)\]$/i", $line, $match)) { // "[nav:<label>]"
                        $label = $match[1];
                        $ini[$label] = [];
                    }
                    elseif(is_array($ini[$label]) && count(explode("=", $line)) > 1) {
                        $line = array_map("trim", explode("=", $line));
                        $key = array_shift($line);
                        $val = join("=", $line);
                        if(in_array($key, array( "item", "type" ))) {
                            $keys = $key."s";
                            if(!is_array($ini[$label][$keys])) { $ini[$label][$keys] = []; };
                            $ini[$label][$keys][] = $val;
                        }
                        else {
                            $ini[$label][$key] = $val;
                        }
                    };
                }
                // ====== Other ======
                else {
                    if(substr($line, 0, 1) == ";") {} // comments;
                    elseif(strtolower($line) == "[".strtolower($type)."]") { // matching section
                        $label = true;
                    }
                    elseif(substr($line, 0, 1) == "[" && substr($line, strlen($line) -1, 1) == "]") { // other section label
                        $label = false;
                    }
                    elseif($label != false && count(explode("=", $line)) > 1) {
                        $item = array_map("trim", explode("=", $line));
                        $key = array_shift($item);
                        $val = join("=", $item);
                        if($type == "hidden") {
                            $ini[] = $val;
                        }
                        else {
                            $ini[$key] = $val;
                        };
                    };
                };
                
            };
            //arrayList($ini);
            return $ini;
        }
        else {
            return false;
        };
    };

// ===================================================
//                    AUTHORIZATION
// ===================================================

    // ====== USER & GROUP authorization / begin ======
    function authorizedIniFile() {
    // ---------------------------------------------
    // Check loggen user authorization & group membership.
    // Put Logged user, group & ini file path into a $_SESSION array
    // ---------------------------------------------
    // RETURNS: <string> valid ini file path or empty string in none found
    // ---------------------------------------------
        // ====== Xable admin panel folder ======
        $url = $_SERVER['REQUEST_URI'];
        if(substr($url, strlen($url) - 1) == "/") { $url = substr($url, 0, strlen($url) - 1); };
        if(!is_string(path($url, "extension")) || path($url, "extension") == "") {
            $_SESSION['admin_folder'] = array_pop(explode("/", $url));
        }
        else {
            $_SESSION['admin_folder'] = array_pop(explode("/", path($url, "dirname")));
        };
        // ====== Check USER / GROUP authorization ======
		$users_folder = "_users";
        $logged_user = $_SERVER['PHP_AUTH_USER'];
        if(is_string($logged_user)) {
            $groups = array_map("trim", file("$users_folder/.groups"));
            foreach($groups as $group) {
                if(substr($group, 0, 1) != ";" && count($group = explode(":", $group)) == 2) {
                    $group_name = $group[0];
                    $group_users = explode(" ", trim($group[1]));
                    if(in_array($logged_user, $group_users)) { $logged_group = $group_name; };
                };
            };
            // ====== OUTPUT ======
            if(file_exists("$users_folder/$logged_group.ini")) {
                $_SESSION['logged_user'] = $logged_user;
                $_SESSION['logged_group'] = $logged_group;
                $_SESSION['ini_file'] = "$users_folder/$logged_group.ini";
            }
            else {
                $_SESSION['logged_user'] = $logged_user;
                $_SESSION['logged_group'] = "none";
                $_SESSION['ini_file'] = "xable.ini";
            };
        }
        else {
            $_SESSION['logged_user'] = "unauthorized";
            $_SESSION['logged_group'] = "none";
            $_SESSION['ini_file'] = "";
        };
        return $_SESSION['ini_file'];
    };
    // ====== USER & GROUP authorization / end ======

?>